# @restroom-mw/core

This package is where the main magic happens.

Listen to endpoints and look for the contracts in the right place.

It manages also the [lifecycle hooks](/architecture?id=lifecycle-hooks) system.

## Usage

Import

```js
const core = require("@restroom-mw/core");
```

and plug

```js
var app = express();
app.use("/api/*", core);
```

## API

<!-- Generated by documentation.js. Update this documentation by updating the source code. -->

#### Table of Contents

-   [Restroom](#restroom)
    -   [Parameters](#parameters)
    -   [setData](#setdata)
        -   [Parameters](#parameters-1)
    -   [getData](#getdata)
        -   [Parameters](#parameters-2)
    -   [onInit](#oninit)
        -   [Parameters](#parameters-3)
    -   [onBefore](#onbefore)
        -   [Parameters](#parameters-4)
    -   [onSuccess](#onsuccess)
        -   [Parameters](#parameters-5)
    -   [onAfter](#onafter)
        -   [Parameters](#parameters-6)
    -   [onError](#onerror)
        -   [Parameters](#parameters-7)
    -   [onException](#onexception)
        -   [Parameters](#parameters-8)
    -   [onFinish](#onfinish)
        -   [Parameters](#parameters-9)

### Restroom

[packages/core/src/restroom.js:13-108](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L13-L108 "Source code on GitHub")

#### Parameters

-   `req` **[object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** express req object
-   `res` **[object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** express res object

**Meta**

-   **copyright**: 2020 Dyne.org

-   **author**: Puria Nafisi Azizi &lt;puria@dyne.org> @pna
-   **license**: AGPL-3.0-only

    Main class that will allow to define function hooks and
    save \`data\` between different middlewares

#### setData

[packages/core/src/restroom.js:25-29](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L25-L29 "Source code on GitHub")

Save a key value, accessible afterwards into the contracts DATA field

##### Parameters

-   `k` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 
-   `v` **[object](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/Object)** 

#### getData

[packages/core/src/restroom.js:36-38](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L36-L38 "Source code on GitHub")

Get the value by key from the DATA field

##### Parameters

-   `k` **[string](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Global_Objects/String)** 

#### onInit

[packages/core/src/restroom.js:51-53](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L51-L53 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onInit lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 

#### onBefore

[packages/core/src/restroom.js:60-62](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L60-L62 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onBefore lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 

#### onSuccess

[packages/core/src/restroom.js:69-71](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L69-L71 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onSuccess lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 

#### onAfter

[packages/core/src/restroom.js:78-80](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L78-L80 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onAfter lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 

#### onError

[packages/core/src/restroom.js:87-89](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L87-L89 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onError lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 

#### onException

[packages/core/src/restroom.js:96-98](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L96-L98 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onException lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 

#### onFinish

[packages/core/src/restroom.js:105-107](https://github.com/dyne/restroom-mw/blob/2070f26ed2bbc93cb0bdb9cf19bfdbdfc0b9ae29/packages/core/src/restroom.js#L105-L107 "Source code on GitHub")

-   **See: [lifecycle](/architecture?id=lifecycle-hooks)
    **

Saves the function to be executed at the onFinish lifecycle step

##### Parameters

-   `fn` **[Function](https://developer.mozilla.org/docs/Web/JavaScript/Reference/Statements/function)** 
